﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Linq.Expressions;
using System.Transactions;
using Volvo.CustomerMaster.Infrastructure.Common.Utilities.Entities;
using Volvo.CustomerMaster.Infrastructure.Common.Utilities.Logging;
using Volvo.CustomerMaster.Infrastructure.Common.Utilities.Security;
using Volvo.CustomerMaster.Infrastructure.Common.Utilities.Specifications;
using Volvo.CustomerMaster.Customer.Model.Common.Utilities.GenericExceptions;
using Volvo.CustomerMaster.Customer.Model.Entities;
using Volvo.CustomerMaster.Customer.Model.RepositoryInterfaces;
using Container = Volvo.CustomerMaster.Infrastructure.Common.Utilities.Unity.Container;
using Volvo.CustomerMaster.Customer.Service;

namespace Volvo.CustomerMaster.Customer.Service
{
    public class CustomerService : ICustomerService 
    {
        public ICustomerRepository CustomerRepository { get; set; }

        public CustomerService()
        {
            CustomerRepository = Container.Resolve<ICustomerRepository>();
        }

        public CustomerService(ICustomerRepository companyRepository)
        {
            CustomerRepository = companyRepository;
        }

        #region ICustomerService Members
               
        public IList<Volvo.CustomerMaster.Customer.Model.Entities.Customer> ListAllOrders(int maxNoOfHits)
        {
            return CustomerRepository.FindAll(maxNoOfHits);
        }

        /// <summary>
        /// Finds the specified filter specification.
        /// </summary>
        /// <param name="filterSpecification">The filter specification.</param>
        /// <param name="sortSpecificationList">The sort specification list.</param>
        /// <returns></returns>
        public IList<Volvo.CustomerMaster.Customer.Model.Entities.Customer> Find(Specification<Volvo.CustomerMaster.Customer.Model.Entities.Customer> filterSpecification, IEnumerable<SortSpecification<Volvo.CustomerMaster.Customer.Model.Entities.Customer>> sortSpecificationList)
        {
            return CustomerRepository.Find(filterSpecification, sortSpecificationList);
        }

        /// <summary>
        /// Finds the specified filter specification.
        /// </summary>
        /// <param name="filterSpecification">The filter specification.</param>
        /// <returns></returns>
        public IList<Volvo.CustomerMaster.Customer.Model.Entities.Customer> Find(Specification<Volvo.CustomerMaster.Customer.Model.Entities.Customer> filterSpecification)
        {
            return CustomerRepository.Find(filterSpecification);
        }

        /// <summary>
        /// Finds orders with filter, sort and paging.
        /// </summary>
        /// <param name="filterSpecification">The filter specification.</param>
        /// <param name="sortSpecificationList">The sort specification list.</param>
        /// <param name="pageNumber">The page number.</param>
        /// <param name="pageSize">Size of the page.</param>
        /// <returns></returns>
        public IList<Volvo.CustomerMaster.Customer.Model.Entities.Customer> Find(Specification<Volvo.CustomerMaster.Customer.Model.Entities.Customer> filterSpecification, IEnumerable<SortSpecification<Volvo.CustomerMaster.Customer.Model.Entities.Customer>> sortSpecificationList, int pageNumber, int pageSize)
        {
            return CustomerRepository.Find(filterSpecification, sortSpecificationList, pageNumber, pageSize);
        }

        /// <summary>
        /// Finds orders with filter, sort, paging and grouping.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="filterSpecification">The filter specification.</param>
        /// <param name="sortSpecificationList">The sort specification list.</param>
        /// <param name="pageNumber">The page number.</param>
        /// <param name="pageSize">Size of the page.</param>
        /// <param name="groupSpecification">The group specification.</param>
        /// <returns></returns>
        public IList<T> Find<T>(Specification<Volvo.CustomerMaster.Customer.Model.Entities.Customer> filterSpecification, IEnumerable<SortSpecification<Volvo.CustomerMaster.Customer.Model.Entities.Customer>> sortSpecificationList, int pageNumber, int pageSize, GroupSpecification<Volvo.CustomerMaster.Customer.Model.Entities.Customer, T> groupSpecification)
        {
            return CustomerRepository.Find(filterSpecification, sortSpecificationList, pageNumber, pageSize, groupSpecification);
        }

        /// <summary>
        /// Counts how many orders the repository has.
        /// </summary>
        /// <returns></returns>
        public int Count()
        {
            return CustomerRepository.Find().Count();
        }

        public int Count(Specification<Volvo.CustomerMaster.Customer.Model.Entities.Customer> filterSpecification)
        {
            return CustomerRepository.Find().Count(filterSpecification.Predicate);
        }       

        #endregion 
    }
}
