﻿using System;
using System.Collections.Generic;
using System.Reflection;
using Microsoft.Practices.EnterpriseLibrary.Validation;

namespace Volvo.CustomerMaster.Infrastructure.Common.Utilities.Entities
{
    [Serializable]
    public abstract class ValidatableObject : BaseObject
    {
        protected ValidationResults validRes;

        /// <summary>
        /// Returns the ValidationResults
        /// </summary>
        public virtual ValidationResults ValidationResults
        {
            get { return validRes; }
        }

        /// <summary>
        /// Validates the entire entity that derives from this class and returns true/false
        /// </summary>
        public virtual bool IsValid
        {
            get
            {
                Validator v = ValidationFactory.CreateValidator(this.GetType());
                validRes = v.Validate(this);
                return validRes.IsValid;
            }
        }

        /// <summary>
        /// Validates the entire entity that derives from this class and the ValidationResults
        /// </summary>
        public virtual ValidationResults Validate
        {
            get
            {
                return ValidationFactory.CreateValidator(this.GetType()).Validate(this);
            }
        }


    }
}
