﻿using System;
using System.Runtime.InteropServices;
using System.Security.Principal;

namespace Volvo.CustomerMaster.Infrastructure.Common.Utilities.Impersonation
{
    public class ImpersonateUser : IDisposable
    {
        [DllImport("advapi32.dll")]
        public static extern int LogonUser(
            String lpszUsername,
            String lpszDomain,
            String lpszPassword,
            uint dwLogonType,
            uint dwLogonProvider,
            out  IntPtr phToken);

        [DllImport("kernel32.dll", SetLastError = true)]
        static extern bool CloseHandle(IntPtr handle);

        private static IntPtr tokenHandle = new IntPtr(0);
        private static WindowsImpersonationContext impersonatedUser;

        enum LogonSessionType : uint
        {
            Interactive = 2,
            Network,
            Batch,
            Service,
            NetworkCleartext = 8,
            NewCredentials
        }

        enum LogonProvider : uint
        {
            Default = 0, // default for platform (use this!)
            WinNT35,     // sends smoke signals to authority
            WinNT40,     // uses NTLM
            WinNT50      // negotiates Kerb or NTLM
        }

        // If you incorporate this code into a DLL, be sure to demand that it
        // runs with FullTrust.
        //[PermissionSetAttribute(SecurityAction.Demand, Name = "FullTrust")]
        public void Impersonate(string domainName, string userName, string password)
        {
            try
            {
                tokenHandle = IntPtr.Zero;

                // ---- Step - 1 
                // Call LogonUser to obtain a handle to an access token.
                int hresult = LogonUser(
                    userName,
                    domainName,
                    password,
                    (uint)LogonSessionType.Network,
                    (uint)LogonProvider.Default,
                    out tokenHandle);         // tokenHandle - new security token

                if (0 == hresult)
                {
                    int ret = Marshal.GetLastWin32Error();
                    //Console.WriteLine("LogonUser call failed with error code : " + ret);
                    throw new System.ComponentModel.Win32Exception(ret);
                }

                // ---- Step - 2 
                WindowsIdentity identity = new WindowsIdentity(tokenHandle);

                // ---- Step - 3 
                impersonatedUser = identity.Impersonate();
            }
            catch (Exception ex)
            {
                Console.WriteLine("Exception occurred. " + ex.Message);
            }
        }

        // Stops impersonation
        public void Undo()
        {
            impersonatedUser.Undo();
            // Free the tokens.
            if (tokenHandle != IntPtr.Zero)
                CloseHandle(tokenHandle);
        }

        #region IDisposable Members

        public void Dispose()
        {
            impersonatedUser.Undo();
            // Free the tokens.
            if (tokenHandle != IntPtr.Zero)
                CloseHandle(tokenHandle);
        }

        #endregion
    }
}
