﻿using System;
using System.Configuration;
using GimXmsApi;
using IBM.XMS;


namespace Volvo.CustomerMaster.Infrastructure.Common.Utilities.Integration
{
    /// <summary>
    /// This class examplifies how to handle initiation of WMQ queues
    /// </summary>
    public abstract class WMQReaderCommunication : IReaderCommunication
    {
        private int _connectionsQty;
        private IGimWmqConnectionInfo[] _connectionInfos;
        private MessageListener[] _messageListeners;
        private ExceptionListener[] _exceptionListeners;
        private IGimWmqReader[] _readers;

        protected WMQReaderCommunication()
        {
            InitializeConnections();
        }

        public void InitializeConnections()
        {
            // Get configurations of queues from configuration file
            WMQueuesConfigurationSection queuesConfig = GetQueuesConfig();

            _connectionsQty = queuesConfig.WMQueues.Count;
            _connectionInfos = new GimWmqConnectionInfo[_connectionsQty];
            _messageListeners = new MessageListener[_connectionsQty];
            _exceptionListeners = new ExceptionListener[_connectionsQty];
            _readers = new GimWmqReader[_connectionsQty];

            int i = 0;
            // Loop through each defined WMQ message queue and setup connection and listeners
            foreach (WMQueueConfig queueConfig in queuesConfig.WMQueues)
            {
                _connectionInfos[i] = new GimWmqConnectionInfo(queueConfig.HostName, queueConfig.QueueManagerName, queueConfig.ChannelName, queueConfig.PortNumber, queueConfig.DestinationQueue);
                _connectionInfos[i].enableTrace();
                _readers[i] = new GimWmqReader(_connectionInfos[i], true);
                _messageListeners[i] = new MessageListener(OnNewMessageCallback);
                _readers[i].SetMessageListener(_messageListeners[i]);
                _exceptionListeners[i] = new ExceptionListener(OnExceptionHandler);
                _readers[i].SetExceptionListener(_exceptionListeners[i]);
                i++;
            }
        }

        public WMQueuesConfigurationSection GetQueuesConfig()
        {
            return ConfigurationManager.GetSection("WMQueuesConfiguration") as WMQueuesConfigurationSection;
        }

        /// <summary>
        /// The concrete class must implement this method to exceptions from WMQ
        /// </summary>
        /// <param name="ex"></param>
        public abstract void OnExceptionHandler(Exception ex);
        /// <summary>
        /// The concrete class must implement this method to receive the messages from WMQ
        /// </summary>
        /// <param name="msg"></param>
        public abstract void OnNewMessageCallback(IMessage msg);

        public void Dispose()
        {
            foreach (IGimWmqReader reader in _readers)
            {
                reader.RemoveMessageListener();
                reader.RemoveExceptionListener();
                reader.Dispose();
            }
        }
    }
}