﻿using System.Configuration;
using GimXmsApi;

namespace Volvo.CustomerMaster.Infrastructure.Common.Utilities.Integration
{
    /// <summary>
    /// This class examplifies how to handle initiation of WMQ queues
    /// </summary>
    public class WMQWriterCommunication : IWriterCommunication
    {
        private int _connectionsQty;
        private IGimWmqConnectionInfo[] _connectionInfos;
        private IGimWmqWriter[] _writers;

        public WMQWriterCommunication()
        {
            InitializeConnections();
        }

        public void InitializeConnections()
        {
            // Get configurations of queues from configuration file
            WMQueuesConfigurationSection queuesConfig = ConfigurationManager.GetSection("WMQueuesConfiguration") as WMQueuesConfigurationSection;

            // Usually one (when sending)
            _connectionsQty = queuesConfig.WMQueues.Count;
            _connectionInfos = new GimWmqConnectionInfo[_connectionsQty];
            _writers = new GimWmqWriter[_connectionsQty];

            int i = 0;
            // Loop through each defined WMQ message queue and setup connection and listeners
            foreach (WMQueueConfig queueConfig in queuesConfig.WMQueues)
            {
                _connectionInfos[i] = new GimWmqConnectionInfo(queueConfig.HostName, queueConfig.QueueManagerName, queueConfig.ChannelName, queueConfig.PortNumber, queueConfig.DestinationQueue);
                _connectionInfos[i].enableTrace();
                _writers[i] = new GimWmqWriter(_connectionInfos[i], true);
                i++;
            }
        }

        public virtual void SendTextMessage(string message)
        {
            _writers[0].SendTextMessage(message);
        }

        public void Dispose()
        {
            foreach (IGimWmqWriter writer in _writers)
            {
                writer.Dispose();
            }
        }
    }
}