﻿using System.Configuration;

namespace Volvo.CustomerMaster.Infrastructure.Common.Utilities.Integration
{
    /// <summary>
    /// Custom configuration section that handles configuration of WMQs used together with e.g. GimXmsToolkit 
    /// </summary>
    public class WMQueuesConfigurationSection : ConfigurationSection
    {
        [
        ConfigurationProperty("WMQueues", IsDefaultCollection = false),
        ConfigurationCollection(typeof(WMQueuesCollection), AddItemName = "WMQueueConfig")
        ]
        public WMQueuesCollection WMQueues
        {
            get { return (WMQueuesCollection) base["WMQueues"]; }
        }
    }

    /// <summary>
    /// Custom configuration element collections that handles configuration of WMQs used together with e.g. GimXmsToolkit 
    /// </summary>
    public class WMQueuesCollection : ConfigurationElementCollection
    {
        protected override ConfigurationElement CreateNewElement()
        {
            return new WMQueueConfig();
        }

        protected override object GetElementKey(ConfigurationElement element)
        {
            return ((WMQueueConfig) element).HostName + ((WMQueueConfig) element).DestinationQueue;
        }
    }

    /// <summary>
    /// Custom configuration elements that handles configuration of WMQs used together with e.g. GimXmsToolkit 
    /// </summary>
    public class WMQueueConfig : ConfigurationElement
    {
        public WMQueueConfig(string hostName, string queueManagerName, string channelName, int portNumber,
                             string destinationQueue)
        {
            HostName = hostName;
            QueueManagerName = queueManagerName;
            ChannelName = channelName;
            PortNumber = portNumber;
            DestinationQueue = destinationQueue;
        }

        /// <summary>
        /// Default constructor, will use default values as defined below.
        /// </summary>
        public WMQueueConfig()
        {
        }

        [ConfigurationProperty("HostName", IsRequired = true, IsKey = true)]
        public string HostName
        {
            get { return (string) this["HostName"]; }
            set { this["HostName"] = value; }
        }

        [ConfigurationProperty("QueueManagerName", IsRequired = true)]
        public string QueueManagerName
        {
            get { return (string) this["QueueManagerName"]; }
            set { this["QueueManagerName"] = value; }
        }

        [ConfigurationProperty("ChannelName", IsRequired = false)]
        public string ChannelName
        {
            get { return (string)this["ChannelName"]; }
            set { this["ChannelName"] = value; }
        }

        [ConfigurationProperty("PortNumber", IsRequired = false)]
        public int PortNumber
        {
            get { return (int) this["PortNumber"]; }
            set { this["PortNumber"] = value; }
        }

        [ConfigurationProperty("DestinationQueue", IsRequired = false)]
        public string DestinationQueue
        {
            get { return (string) this["DestinationQueue"]; }
            set { this["DestinationQueue"] = value; }
        }
    }
}