﻿using System.Collections.Generic;
using System.Diagnostics;
using Microsoft.Practices.EnterpriseLibrary.Logging;

namespace Volvo.CustomerMaster.Infrastructure.Common.Utilities.Logging
{
    public class ContextInfo : Dictionary<string, object>
    {
        public ContextInfo() : base() { }
    }

    /// <summary>
    /// Example of a Wrapper class for the logging functionality of EntLib
    /// </summary>
    public static class GeneralLogger
    {
        #region ILogger Members

        public static void LogUseCaseStep(string message)
        {
            LogEntry logEntry = new LogEntry(message, Properties.Settings.Default.LogCategoryUCStep, 1, 0, TraceEventType.Information, "", null);
            Logger.Write(logEntry);
        }

        public static void LogDecision(string message, object decision)
        {
            ContextInfo ctx = new ContextInfo();
            ctx.Add(Properties.Settings.Default.LogCategoryDecision, decision);
            LogEntry logEntry = new LogEntry(message, Properties.Settings.Default.LogCategoryDecision, 1, 0, TraceEventType.Information, "", ctx);
            Logger.Write(logEntry);
        }

        public static void LogBusinessRuleResult(string message, IDictionary<string, object> properties)
        {
            LogEntry logEntry = new LogEntry(message, Properties.Settings.Default.LogCategoryBRResult, 1, 0, TraceEventType.Information, "", properties);
            Logger.Write(logEntry);
        }

        public static void LogInfo(string message)
        {
            LogEntry logEntry = new LogEntry(message, Properties.Settings.Default.LogCategoryInfo, 1, 0, TraceEventType.Information, "", null);
            Logger.Write(logEntry);
        }

        public static void LogError(string message)
        {
            LogEntry logEntry = new LogEntry(message, Properties.Settings.Default.LogCategoryError, 1, 0, TraceEventType.Error, "", null);
            Logger.Write(logEntry);
        }

        public static void LogWarning(string message)
        {
            LogEntry logEntry = new LogEntry(message, Properties.Settings.Default.LogCategoryWarning, 2, 0, TraceEventType.Warning, "", null);
            Logger.Write(logEntry);
        }

        public static void LogDebug(string message)
        {
            LogEntry logEntry = new LogEntry(message, Properties.Settings.Default.LogCategoryDebug, 3, 0, TraceEventType.Verbose, "", null);
            Logger.Write(logEntry);
        }

        public static void LogTrace(string message)
        {
            LogEntry logEntry = new LogEntry(message, Properties.Settings.Default.LogCategoryTrace, 4, 0, TraceEventType.Verbose, "", null);
            Logger.Write(logEntry);
        }

        public static void LogGeneralMessage(string message, string category, TraceEventType eventType)
        {
            LogEntry logEntry = new LogEntry(message, category, 0, 0, eventType, "", null);
            Logger.Write(logEntry);
        }

        #endregion
    }

}
