﻿using System;
using System.ServiceModel;
using NHibernate;
using NHibernate.Context;
using Volvo.CustomerMaster.Infrastructure.Common.Utilities.WCF;

[assembly: log4net.Config.XmlConfigurator(Watch = true)]

namespace Volvo.CustomerMaster.Infrastructure.Common.Utilities.Repositories
{
    /// <summary>
    /// Is respoonsible for session management.
    /// </summary>
    
    public class SessionProvider
    {
        // Configure log4net using the .config file
        private static ISessionFactory sessionFactory;
        public static NHibernate.Cfg.Configuration configuration;

        static SessionProvider()
        {
            // Initialize
            // Note: Set the BuildAction property of each mapping file to "Embedded Resource"
            configuration = new NHibernate.Cfg.Configuration().Configure();
            // Create session factory from configuration object
            sessionFactory = configuration.BuildSessionFactory();
        }

        /// <summary>
        /// Returns singleton factory created from 1) app.config or 2) hibernate.cfg.xml
        /// </summary>
        /// <returns></returns>
        public static ISessionFactory GetSessionFactory()
        {
            return sessionFactory;
        }

        /// <summary>
        /// Get new session
        /// </summary>
        /// <returns></returns>
        public static ISession GetNewSession()
        {
            return sessionFactory.OpenSession();
        }

        /// <summary>
        /// Get new or open session from the session factory
        /// </summary>
        /// <returns></returns>
        public static ISession GetNewOrCurrentSession()
        {
            ISession session = null;

            #region Used only for WCF
            // Used only for WCF
            if (OperationContext.Current != null)
            {
                var contextManager = OperationContext.Current.InstanceContext.Extensions.Find<NHibernateContextExtension>();
                if (contextManager == null)
                {
                    throw new InvalidOperationException(@"There is no context manager available.");
                }
                session = contextManager.Session;
                return session;
            }
            #endregion

            if (!CurrentSessionContext.HasBind(sessionFactory))
            {
                session = sessionFactory.OpenSession();
                CurrentSessionContext.Bind(session);
            }
            else
            {
                session = sessionFactory.GetCurrentSession();
                if (!session.IsOpen)
                    session = sessionFactory.OpenSession();
            }
            return session;
        }

    }

}

