﻿using System.Collections.Generic;

namespace Volvo.CustomerMaster.Infrastructure.Common.Utilities.Translators
{
    /// <summary>
    /// Base class for handling translations between types
    /// </summary>
    /// <typeparam name="TEntity"></typeparam>
    /// <typeparam name="TService"></typeparam>
    public abstract class BaseEntityTranslator<TEntity, TService> : IEntityTranslator<TEntity, TService>
    {
        public abstract TEntity ToEntity(TService obj);

        public IList<TEntity> ToEntity(IEnumerable<TService> services)
        {
            IList<TEntity> entities = new List<TEntity>();
            foreach (TService service in services)
                entities.Add(this.ToEntity(service));
            return entities;
        }

        public abstract TService ToService(TEntity obj);

        public IList<TService> ToService(IEnumerable<TEntity> entities)
        {
            IList<TService> services = new List<TService>();
            foreach (TEntity entity in entities)
                services.Add(this.ToService(entity));
            return services;
        }
    }
}
