﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Security.Principal;
using Volvo.CustomerMaster.Infrastructure.Common.Utilities.ValueObjects;

namespace Volvo.CustomerMaster.Infrastructure.Common.Utilities.Entities
{
    public class GenericUser : GenericEntity
    {

        public GenericUser()
        {
            Roles = new Collection<GenericUserRole>();
        }

        #region Properties

        public GenericPrincipal Principal 
        {
            get
            {
                if (UserVCN != "" && Roles.Count > 0)
                {
                    return new GenericPrincipal(new GenericIdentity(UserVCN), rolesToArray());
                }
                return null;
            }
            //set; 
        }

        /// <summary>
        /// VCN id for a user (e.g v038723)
        /// </summary>
        public string UserVCN { get; set; }

        Collection<GenericUserRole> roles;
        public Collection<GenericUserRole> Roles
        {
            get { return roles; }
            private set { roles = value; }
        }

        #endregion

        #region Methods


        /// <summary>
        /// Add role to user
        /// </summary>
        /// <param name="role"></param>
        public void AddRole(GenericUserRole role)
        {
            roles.Add(role);
        }

        /// <summary>
        /// Check if the user has the role
        /// </summary>
        /// <param name="roleName"></param>
        /// <returns></returns>
        public bool HasRole(GenericUserRole userRole)
        {
            if (roles != null)
            {
                foreach (var role in roles)
                {
                    if (role.Name.Equals(userRole.Name, StringComparison.CurrentCultureIgnoreCase))
                        return true;
                }
            }
            return false;
        }

        #endregion

        #region Private Methods

        private string[] rolesToArray()
        {
            List<string> roles = new List<string>();
            foreach (GenericUserRole role in Roles)
            {
                roles.Add(role.Name);
            }
            return roles.ToArray();
        }
        #endregion
    }
}
