﻿using System;
using NHibernate;
using System.ServiceModel.Description;
using System.ServiceModel.Dispatcher;
using System.ServiceModel.Channels;

namespace Volvo.CustomerMaster.Infrastructure.Common.Utilities.WCF
{
    /// <summary>
    /// With a way to attach a new ISession to every InstanceContext that is created, 
    /// we need to trigger this process on each and every call.  
    /// To do that we will attribute the WCF service with a custom attribute. 
    /// </summary>
    public class NHibernateWCFContextAttribute : Attribute, IContractBehavior
    {
        public ISessionFactory SessionFactory { private get; set; }

        public void Validate(ContractDescription contractDescription,
                                ServiceEndpoint endpoint)
        { }

        /// <summary>
        /// This method is called once for each endpoint that uses the specified service contract.
        /// </summary>
        /// <param name="contractDescription"></param>
        /// <param name="endpoint"></param>
        /// <param name="dispatchRuntime"></param>
        public void ApplyDispatchBehavior(ContractDescription contractDescription,
                                ServiceEndpoint endpoint,
                                DispatchRuntime dispatchRuntime)
        {
            dispatchRuntime.InstanceContextInitializers.Add(
                new NHibernateContextInitializer());
        }

        public void ApplyClientBehavior(ContractDescription contractDescription,
                                ServiceEndpoint endpoint,
                                ClientRuntime clientRuntime)
        { }

        public void AddBindingParameters(ContractDescription contractDescription,
                                ServiceEndpoint endpoint,
                                BindingParameterCollection bindingParameters)
        { }
    }
}
