﻿using System;
using System.Xml.Schema;
using System.Xml;
using System.Xml.Linq;
using System.IO;

namespace Volvo.CustomerMaster.Infrastructure.Common.Utilities.Xml
{
    /// <summary>
    /// Example of an xml validator that validates against a xsd schema
    /// </summary>
    public class XmlValidator : XmlSchemaSet, IDisposable
    {
        private StreamReader reader;

        /// <summary>
        /// Validates an xml document against a xml schema
        /// </summary>
        /// <param name="xmlString"></param>
        /// <param name="xsdStream"></param>
        public void ValidateXml(string xmlString, Stream xsdStream)
        {
            XDocument xdoc = XDocument.Parse(xmlString, LoadOptions.PreserveWhitespace);

            // Open the target schema file as a stream.
            using (reader = new StreamReader(xsdStream))
            {
                ValidationEventHandler += xmlSchemaSet_ValidationEventHandler;
                var xmlSchema = XmlSchema.Read(reader, xmlSchemaSet_ValidationEventHandler);
                Add(xmlSchema);
                reader.Close();
            }

            var validationError = false;
            var errMsg = "";
            // Validate the xml against the xml schema
            xdoc.Validate(this, (o, args) =>
            {
                errMsg += string.Format("\nXML validation error: {0}", args.Message);
                validationError = true;
            });
            if (validationError)
            {
                throw new XmlException(errMsg);
            }
        }

        void xmlSchemaSet_ValidationEventHandler(object sender, ValidationEventArgs e)
        {
            throw new Exception(e.Message);
        }

        #region IDisposable Members

        public void Dispose()
        {
            if (reader != null)
            {
                reader.Close();
            }
        }

        #endregion
    }
}
