﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
using System.Text;
using System.Web.Mvc;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using Volvo.CustomerMaster.Web.Common.Security;
using System.Linq;
namespace Volvo.CustomerMaster.Web.Common.Helpers
{
    /// <summary>
    /// This class contain Volvo Custom HtmlHelper extensions for Buttons/images. 
    /// Import the namespace to your aspx-view and you will get these in intellicence 
    /// </summary>
    public static class ButtonHelper
    {
        private static IEnumerable<KeyValuePair<string, object>> GetAttributeValues(object attributeValues)
        {
            if (attributeValues != null)
            {
                Type type = attributeValues.GetType();
                //PropertyInfo[] properties = type.GetProperties();
                foreach (PropertyInfo property in type.GetProperties())
                {
                    string attributeName = property.Name;
                    object value = property.GetValue(attributeValues, null);
                    yield return new KeyValuePair<string, object>(attributeName, value);
                }
            }
        }


        /// <summary>   
        /// Renders an HTML form submit button including authorization  
        /// </summary>   
        public static string Button(this HtmlHelper helper, string name, string buttonText)
        {
            return Button(helper, name, buttonText, false,null);
        }

        public static string Button(this HtmlHelper helper, string name, string buttonText, object htmlAttributes)
        {
            return Button(helper, name, buttonText, false, GetAttributeValues(htmlAttributes));
        }

        public static string Button(this HtmlHelper helper, string name, string buttonText, bool disabled, object  htmlAttributes)
        {
            return Button(helper, name, buttonText, disabled, GetAttributeValues(htmlAttributes));
        }

        private static string Button(this HtmlHelper helper, string name, string buttonText, bool disabled, IEnumerable<KeyValuePair<string ,object >> htmlAttributes)
        {
            HtmlGenericControl a = new HtmlGenericControl("input");
            a.ID = name;
            a.Attributes["name"] = name;
            a.Attributes["value"] = buttonText;
            a.Attributes["type"] = "button";
            if (disabled)
                a.Attributes["disabled"] = "disabled";
            if (htmlAttributes != null)
                foreach (KeyValuePair<string, object> attribute in htmlAttributes)
                {
                    a.Attributes[attribute.Key] = attribute.Value.ToString();
                }

            StringBuilder htmlBuilder = new StringBuilder();
            HtmlTextWriter htmlWriter = new HtmlTextWriter(new StringWriter(htmlBuilder));
            // render the html
            a.RenderControl(htmlWriter);
            string html = htmlBuilder.ToString();//= String.Format("<input type=\"submit\" name=\"{0}\" value=\"{1}\" ", name, buttonText);
            //html += (disabled) ? " disabled=\"disabled\" />" : "/>";
            return html;
        }
        /// <summary>   
        /// Renders an HTML form image button   
        /// </summary>   
        public static string AddFilesButton(this HtmlHelper helper, string imageURI)
        {
            if (string.IsNullOrEmpty(imageURI))
                imageURI = helper.ViewContext.HttpContext.Request.ApplicationPath +
                           "../../Content/Images/Volvo Icons/Add Files/Add_16x16.png";
             return String.Format("<input type=\"image\" src=\"{0}\" />", imageURI);
        }

        public static bool SetButtonDisability( string id, string rule)
        {
            try
            {
                if (SecurityHelper.Authorized(rule))
                {
                    return false;
                }
            }
            catch
            { }
            return true;
        }
    }
}

