﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Resources;
using System.Text;
using System.Threading;
using System.Web;
using System.Web.Configuration;
using System.Web.Mvc;
using Volvo.CustomerMaster.Web;

namespace Volvo.CustomerMaster.Web.Common.Helpers
{
    public static class LocalizationHelper
    {
        private static string _lastCultureName = string.Empty;
        private static readonly StringBuilder JavascriptValues = new StringBuilder();

        static LocalizationHelper()
        {
            var defaultConfig = ConfigurationManager.GetSection("system.web/globalization") as GlobalizationSection;
            if (defaultConfig != null)
                _lastCultureName = defaultConfig.Culture;

        }

        /// <summary>
        /// Helper to gets the assembly information from a View.
        /// </summary>
        /// <param name="instance">The instance.</param>
        public static MvcHtmlString GetLocalizedJavascriptValues(this HtmlHelper instance)
        {
            if (!Thread.CurrentThread.CurrentCulture.Name.Equals(_lastCultureName))
            {
                _lastCultureName = Thread.CurrentThread.CurrentCulture.Name;

                var defaultConfig = ConfigurationManager.GetSection("system.web/globalization") as GlobalizationSection;

                if (defaultConfig != null && _lastCultureName == defaultConfig.Culture)
                {
                    JavascriptValues.Remove(0, JavascriptValues.Length - 1);
                }
                else
                {
                    JavascriptValues.Clear();

                    JavascriptValues.Append(
                        "if ($.NVS == undefined) $.NVS = {}; " +
                        "$.extend($.NVS, { " +
                            "localization: { ");

                    var sourceType = typeof(Telerik_Resources);
                    foreach (var property in sourceType.GetProperties().Where(property => property.Name.StartsWith("Javascript_") && !property.Name.Equals("Culture") && !property.Name.Equals("ResourceManager")))
                    {
                        JavascriptValues.AppendFormat(
                            "{0}: '{1}',",
                            property.Name.Substring("Javascript_".Length),
                            property.GetValue(property, null)
                            );
                    }

                    JavascriptValues.Remove(JavascriptValues.Length - 1, 1);
                    JavascriptValues.Append( 
                            "} " +
                        "});\n");
                }
            }

            return MvcHtmlString.Create(JavascriptValues.ToString());
        }
    }
}