﻿using System;
using System.Web;
using System.Web.Mvc;

namespace Volvo.CustomerMaster.Web.Common.Helpers
{
    /// <summary>
    /// Gathers extensions related to Themes handling
    /// </summary>
    public static class ThemesHelper
    {
        private const string SelectedBrandCookie = "selectedBrand";
        /// <summary>
        /// Gets the current theme.
        /// </summary>
        /// <param name="instance">The <see cref="HtmlHelper"/> instance.</param>
        /// <returns>current theme name</returns>
        public static string GetCurrentTheme(this HtmlHelper instance)
        {
            var cookie = instance.ViewContext.HttpContext.Request.Cookies[SelectedBrandCookie] ?? new HttpCookie(SelectedBrandCookie);

            if (string.IsNullOrEmpty(cookie.Value))
            {   // defaults
                cookie.HttpOnly = true;
                cookie.Expires = DateTime.MaxValue;
                cookie.Path = "/";
                cookie.Value = "Violin";
                instance.ViewContext.HttpContext.Response.SetCookie(cookie);
            }

            return cookie.Value + "/";
        }

        /// <summary>
        /// Sets the current theme.
        /// </summary>
        /// <param name="themeName">Name of the theme.</param>
        /// <param name="httpContextBase">The HTTP context base.</param>
        public static void SetCurrentTheme(string themeName, HttpContextBase httpContextBase)
        {
            var cookie = httpContextBase.Request.Cookies[SelectedBrandCookie] ?? new HttpCookie(SelectedBrandCookie);

            cookie.HttpOnly = true;
            cookie.Expires = DateTime.MaxValue;
            cookie.Path = "/";
            cookie.Value = themeName;

            httpContextBase.Response.SetCookie(cookie);
        }
    }
}
