﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.Mvc;
using System.Web.Mvc.Html;
using Microsoft.Practices.EnterpriseLibrary.Validation;
using Volvo.CustomerMaster.Web.Common.Extensions;

namespace Volvo.CustomerMaster.Web.Common.Helpers
{
    public static class ValidationHelper
    {
        /// <summary>
        /// Displays the validation summary together with flash messages
        /// </summary>
        /// <seealso cref="FlashMessageControllerExtension"/>
        /// <param name="htmlHelper">The HTML helper.</param>
        /// <param name="htmlAttributes">additional HTML attributes.</param>
        /// <returns>rendered validation and messages section</returns>
        public static MvcHtmlString VolvoValidationSummary(this HtmlHelper htmlHelper, IDictionary<string, object> htmlAttributes)
        {
            if (htmlHelper == null)
                throw new ArgumentNullException("htmlHelper");

            var maxMessagePriority = MessageType.Info;
            var htmlSummary = new StringBuilder();

            /*
            if (!htmlHelper.ViewData.ModelState.IsValid)
            {
                htmlSummary.Append(htmlHelper.ValidationSummary(false));
                maxMessagePriority = MessageType.Error;
            }*/

            // add flash messages if needed
            var messenger = (FlashMessenger)htmlHelper.ViewContext.HttpContext.Session[FlashMessenger.SessionKey];

            if (messenger != null)
            {
                if (htmlSummary.ToString().Contains("validation-summary-errors"))
                {
                    var indexFinalList = htmlSummary.ToString().IndexOf("</ul>") - 1;

                    foreach (var kvp in messenger.Messages.Where(kvp => kvp.Value.Count > 0))
                    {
                        while (kvp.Value.Count > 0)
                        {
                            htmlSummary.Insert(indexFinalList, "<li>" + kvp.Value.Dequeue() + "</li>");
                        }
                    }
                }
                else
                {
                    bool itemInserted = false;

                    foreach (var kvp in messenger.Messages.Where(kvp => kvp.Value.Count > 0))
                    {
                        if ((int)maxMessagePriority < (int)kvp.Key)
                            maxMessagePriority = kvp.Key;
                        htmlSummary.AppendLine("<ul class=\"" + kvp.Key.ToString().ToLower() + "\">");
                        while (kvp.Value.Count > 0)
                        {
                            htmlSummary.AppendLine("<li>" + kvp.Value.Dequeue() + "</li>");
                            itemInserted = true;
                        }
                        htmlSummary.AppendLine("</ul>");
                    }

                    if (itemInserted)
                    {
                        htmlSummary.Insert(0, string.Format("<div id=\"messages\" class=\"message {0}\">", maxMessagePriority.ToString().ToLower()));
                        htmlSummary.AppendLine("</div>");
                    }
                }
            }

            return MvcHtmlString.Create(htmlSummary.ToString());
        }

        /// <summary>
        /// Displays the validation summary together with flash messages
        /// </summary>
        /// <seealso cref="FlashMessageControllerExtension"/>
        /// <param name="htmlHelper">The HTML helper.</param>
        /// <returns>rendered validation and messages section</returns>
        public static MvcHtmlString VolvoValidationSummary(this HtmlHelper htmlHelper)
        {
            return VolvoValidationSummary(htmlHelper, null);
        }

        /// <summary>
        /// Copies the only new errors to model stae.
        /// </summary>
        /// <param name="modelStateDictionary">The model state dictionary.</param>
        /// <param name="validationResults">The validation results.</param>
        public static void CopyOnlyNewErrorsToModelState(this ModelStateDictionary modelStateDictionary, ValidationResults validationResults)
        {
            foreach (var validation in validationResults)
            {
                var alreadyExistValidation = false;

                foreach (var stateValues in modelStateDictionary.TakeWhile(stateValues => !alreadyExistValidation))
                {
                    alreadyExistValidation = stateValues.Value.Errors.Any(errors => errors.ErrorMessage == validation.Message);
                }

                if (!alreadyExistValidation)
                {
                    modelStateDictionary.AddModelError(validation.Target.GetType().Name + "." + validation.Key, validation.Message);
                }
            }
        }
    }
}