﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Web;
using System.Web.Mvc;
namespace Volvo.CustomerMaster.Web.Common.Helpers
{
    public static class VersionHelper
    {
        #region Fields
        private static string _formatedVersionDate;
        private static string _buildVersion;
        #endregion

        #region Properties
        /// <summary>
        /// Gets the formated version date.
        /// </summary>
        /// <value>The formated version date.</value>
        public static string FormatedVersionDate
        {
            get
            {
                return _formatedVersionDate ?? (_formatedVersionDate = GetVersionDate());
            }
        }

        /// <summary>
        /// Gets the build version.
        /// </summary>
        /// <value>The build version.</value>
        public static string BuildVersion
        {
            get
            {
                return _buildVersion ?? (_buildVersion = GetBuildVersion());
            }
        }
        #endregion

        #region Commun Functions

        /// <summary>
        /// Gets the build date.
        /// </summary>
        /// <returns></returns>
        public static string GetVersionDate()
        {
            var assemblyFile = Assembly.GetExecutingAssembly().Location;
            var creationTime = System.IO.File.GetCreationTime(assemblyFile);
            return creationTime.ToShortDateString(); // "2007-04-24";
        }
        /// <summary>
        /// Gets the build version of current assembly.
        /// </summary>
        /// <returns></returns>
        public static string GetBuildVersion()
        {
            return GetBuildVersion(Assembly.GetExecutingAssembly());
        }
        /// <summary>
        /// Gets the build version.
        /// </summary>
        /// <param name="a">Assembly</param>
        /// <returns></returns>
        public static string GetBuildVersion(Assembly a)
        {
            var v = a.GetName().Version;
            var versionString = String.Format("{0}.{1}.{2}.{3}", v.Major, v.Minor, v.MajorRevision, v.MinorRevision);
            return versionString;
        }

        /// <summary>
        /// Gets the name of the build.
        /// </summary>
        /// <param name="a">assembly to use</param>
        /// <returns></returns>
        public static string GetBuildName(Assembly a)
        {
            //we configured build system to put build name in description of assemblies
            var desc = (Attribute.GetCustomAttribute(a, typeof(AssemblyDescriptionAttribute)) as AssemblyDescriptionAttribute);
            return desc != null ? desc.Description : a.GetName().Version.ToString();
        }

        /// <summary>
        /// Gets the name of the build.
        /// </summary>
        /// <returns></returns>
        public static string GetBuildName()
        {
            return GetBuildName(Assembly.GetCallingAssembly());
        }

        /// <summary>
        /// Gets the version.
        /// </summary>
        /// <returns></returns>
        public static Version GetVersion()
        {
            return Assembly.GetCallingAssembly().GetName().Version;
        }
        #endregion

        #region MVC Helpers
        /// <summary>
        /// Helper to gets the assembly information from a View.
        /// </summary>
        /// <param name="instance">The instance.</param>
        public static MvcHtmlString GetAssemblyInformation(this HtmlHelper instance)
        {
            return MvcHtmlString.Create(string.Format("{0} - {1}", BuildVersion, FormatedVersionDate));
        }
        #endregion
    }
}