﻿using System;
using System.Web;
using Volvo.CustomerMaster.Web.Common.Session;
using System.Web.Mvc;
using Microsoft.Practices.EnterpriseLibrary.Security;

namespace Volvo.CustomerMaster.Web.Common.Security
{
    /// <summary>
    /// Attribute use for authorizing access to MVC actions. Example: [RuleAuthorize(Allow="IsAdministrator,IsDataSteward")]
    /// </summary>
    public class RuleAuthorizeAttribute : System.Web.Mvc.AuthorizeAttribute
    {
        public String Allow;
        protected bool FailedRolesAuth = false;

        public RuleAuthorizeAttribute()
        {
        }

        public RuleAuthorizeAttribute(String allowedRules)
        {
            Allow = allowedRules;
        }

        protected override bool AuthorizeCore(HttpContextBase httpContext)
        {
            if (SessionWrapper.CurrentUser == null)
            {
                if (!SecurityHelper.Authenticate())
                {
                    return false;
                }
            }

            else if (Allow == null)
            {
                return true;
            }

            if (httpContext == null)
            {
                throw new ArgumentNullException("httpContext");
            }

            if (!SessionWrapper.CurrentUser.Principal.Identity.IsAuthenticated)
            {
                return false;
            }

            try
            {
                string[] allowRoles = Allow.Split(',');
                foreach (string role in allowRoles)
                {
                    string userRole = role.Trim();

                    IAuthorizationProvider auth = AuthorizationFactory.GetAuthorizationProvider("RulesProvider");
                    // Authorize user (with its roles) agains the rule
                    if (auth.Authorize(SessionWrapper.CurrentUser.Principal, userRole))
                    {
                        return true;
                    }
                }
            }
            catch (Exception)
            {
                FailedRolesAuth = true;
                return false;
            }
            FailedRolesAuth = true;
            return false;

        }

        /// <summary>
        /// Method invokes base.OnAuthorization and in addition redirect user to Shared/NotAuth view
        /// when authorization failed.
        /// </summary>
        public override void OnAuthorization(AuthorizationContext filterContext)
        {
            base.OnAuthorization(filterContext);
            if (FailedRolesAuth)
            {
                filterContext.Result = new ViewResult { ViewName = "NotAuthorized" };
            }
        }
    }
}
