﻿using System.Web;
using Volvo.CustomerMaster.Infrastructure.Common.Utilities.Entities;

namespace Volvo.CustomerMaster.Web.Common.Session
{
    /// <summary>
    /// This class wraps the httpContext session into usable methods
    /// </summary>
    public static class SessionWrapper
    {
        private const string StrCurrentUser = "CurrentUser";

        /// <summary>
        /// Get current user from session
        /// </summary>
        public static GenericUser CurrentUser
        {
            get { return GetFromSession<GenericUser>(StrCurrentUser); }
            set { SetInSession(StrCurrentUser, value); }
        }

        #region private members
        /// <summary>
        /// Get object from session
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="key"></param>
        /// <returns></returns>
        private static T GetFromSession<T>(string key)
        {
            object obj = HttpContext.Current.Session[key];
            return obj == null ? default(T) : (T)obj;
        }
        /// <summary>
        /// Set object in session
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="key"></param>
        /// <param name="value"></param>
        private static void SetInSession<T>(string key, T value) where T : class
        {
            if (value == null)
                HttpContext.Current.Session.Remove(key);
            else
            {
                HttpContext.Current.Session[key] = value;
            }
        }
        #endregion

    }
}
