﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Web;
using System.Web.Mvc;

namespace Volvo.CustomerMaster.Web.Common.Validation
{
    public class DateTimeRangeAttribute : ValidationAttribute, IClientValidatable, IMetadataAware
    {
        private DateTime _minValue;
        private DateTime _maxValue;

        /// <summary>
        /// Gets or sets the min value.
        /// </summary>
        /// <value>The min value.</value>
        public string MinValue
        {
            get { return _minValue.ToString(); }
            set { _minValue = DateTime.Parse(value); }
        }

        /// <summary>
        /// Gets or sets the max value.
        /// </summary>
        /// <value>The max value.</value>
        public string MaxValue
        {
            get { return _maxValue.ToString(); }
            set { _maxValue = DateTime.Parse(value); }
        }

        public override bool IsValid(object value)
        {
            if (value == null || !(value is DateTime))
                return false;

            var dateValue = (DateTime)value;
            return _minValue <= dateValue && dateValue <= _maxValue;
        }

        public IEnumerable<ModelClientValidationRule> GetClientValidationRules(ModelMetadata metadata, ControllerContext context)
        {
            return new[]
                       {
                           new ModelClientValidationDataTimeRangeRule(FormatErrorMessage(metadata.GetDisplayName()), _minValue, _maxValue)
                       };
        }

        public void OnMetadataCreated(ModelMetadata metadata)
        {
            metadata.DataTypeName = "DateTime";
        }
    }
}