﻿using System.Web.Mvc;
using Microsoft.Practices.EnterpriseLibrary.Validation;
using Volvo.CustomerMaster.Infrastructure.Common.Utilities.Logging;

namespace Volvo.CustomerMaster.Web.Common.Validation
{
    /// <summary>
    /// Extension class for the ModelStateDictionary
    /// </summary>
    public static class ModelStateDictionaryExtension
    {
        /// <summary>
        /// Adds an error to modelstate and also logs the error using logging application block
        /// </summary>
        /// <param name="modelState"></param>
        /// <param name="key"></param>
        /// <param name="message"></param>
        /// <param name="value"></param>
        public static void AddModelErrorWithLogging(this ModelStateDictionary modelState,
                                                    string key, string message, ValueProviderResult value)
        {

            modelState.AddModelError(key ?? "_FORM", message);
            modelState.SetModelValue(key ?? "_FORM", value);

            // Note: this is a simple example. In real application extend this method to enable more detailed logging
            GeneralLogger.LogGeneralMessage(message, "Debug", System.Diagnostics.TraceEventType.Verbose);
        }

        /// <summary>
        /// Adds an error (not connected to a specific attribute) to modelstate and 
        /// also logs the error using logging application block
        /// </summary>
        /// <param name="modelState"></param>
        /// <param name="message"></param>
        public static void AddModelErrorWithLogging(this ModelStateDictionary modelState,
                                                    string message)
        {
            ValueProviderResult v = new ValueProviderResult(null, "", null);
            modelState.AddModelError("na" ?? "_FORM", message);
            modelState.SetModelValue("na" ?? "_FORM", v);

            // Note: this is a simple example. In real application extend this method to enable more detailed logging
            GeneralLogger.LogGeneralMessage(message, "Debug", System.Diagnostics.TraceEventType.Verbose);
        }
    } 
}
