﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Volvo.CustomerMaster.Web.Models;
using Volvo.CustomerMaster.Customer.Service;
using Volvo.CustomerMaster.Web.Common.Helpers;
using Telerik.Web.Mvc;
using AutoMapper;
using Telerik.Web.Mvc.Infrastructure;
using Microsoft.Practices.EnterpriseLibrary.PolicyInjection;
using Volvo.CustomerMaster.Infrastructure.Common.Utilities.Unity;
using Volvo.CustomerMaster.Web.Common.Security;

namespace Volvo.CustomerMaster.Web.Controllers
{
    [HandleError]
    [RuleAuthorize(Allow="IsDataSteward")]
    public class CustomerController : BaseController
    {
        public ICustomerService CustomerService { get; set; }

        public CustomerController(ICustomerService customerService)
        {
            CustomerService = customerService;  
            
        }

        public CustomerController()
        {
            CustomerService = PolicyInjection.Wrap<ICustomerService>(Container.Resolve<CustomerService>());
        }
                
        public ActionResult Index()
        {
            return View("ListCustomers", new CustomerPresentationModel() { CustomerCount = CustomerService.Count() });
        }

        [GridAction(EnableCustomBinding = true)]
        public ActionResult ListCustomerAjax(GridCommand command)
        {
            var filterSpecification = TelerikHelper.TranslateFilter<Volvo.CustomerMaster.Customer.Model.Entities.Customer>(command.FilterDescriptors);
            var sortSpecificationList = TelerikHelper.TranslateCustomerSort(command.GroupDescriptors, command.SortDescriptors);
            var groupSpecification = TelerikHelper.TranslateCustomerGroup(command.GroupDescriptors);
            var total = (filterSpecification == null) ? CustomerService.Count() : CustomerService.Count(filterSpecification);

            if (groupSpecification == null)
            {
                var domainCustomers = CustomerService.Find(filterSpecification, sortSpecificationList, command.Page - 1, command.PageSize);
                var customers = Mapper.Map<IList<Volvo.CustomerMaster.Customer.Model.Entities.Customer>, IList<Volvo.CustomerMaster.Web.Models.Entities.Customer>>(domainCustomers);

                return View(new GridModel
                {
                    Data = customers,
                    Total = total
                });
            }
            else
            {
                var domainGroupCustomers = CustomerService.Find(filterSpecification, sortSpecificationList, command.Page, command.PageSize, groupSpecification);
                var entityGroup = TranslateDomainGroupToEntityGroup(domainGroupCustomers);

                return View(new GridModel
                {
                    Data = entityGroup,
                    Total = total
                });
            }
        }

        private static IEnumerable<AggregateFunctionsGroup> TranslateDomainGroupToEntityGroup(IEnumerable<AggregateFunctionsGroup> groups)
        {
            foreach (var customerGroup in groups)
            {
                if (customerGroup.Items is IEnumerable<AggregateFunctionsGroup>)
                {
                    customerGroup.Items = TranslateDomainGroupToEntityGroup(customerGroup.Items as IEnumerable<AggregateFunctionsGroup>);
                }
                else if (customerGroup.Items is List<Volvo.CustomerMaster.Customer.Model.Entities.Customer>)
                {
                    customerGroup.Items = Mapper.Map<IList<Volvo.CustomerMaster.Customer.Model.Entities.Customer>, IList<Volvo.CustomerMaster.Web.Models.Entities.Customer>>(customerGroup.Items as List<Volvo.CustomerMaster.Customer.Model.Entities.Customer>);
                }
            }
            return groups;
        }

    }
}
