﻿using System;
using System.Web.Mvc;
using System.Web.Routing;
using System.Web.UI;
using NHibernate;
using NHibernate.Context;
using Volvo.CustomerMaster.Infrastructure.Common.Utilities.Repositories;
using Volvo.CustomerMaster.Web.Common.Security;
using Volvo.CustomerMaster.Web.Common.GenericExceptions;
using Volvo.CustomerMaster.Web.Common.Validation;
using Volvo.CustomerMaster.Web.Models.Translators;
using Telerik.Web.Mvc;
using System.Web;
using Volvo.CustomerMaster.Web.Common.Helpers;

namespace Volvo.CustomerMaster.Web
{
    // Note: For instructions on enabling IIS6 or IIS7 classic mode, 
    // visit http://go.microsoft.com/?LinkId=9394801

    public class MvcApplication : System.Web.HttpApplication
    {
        public static void RegisterGlobalFilters(GlobalFilterCollection filters)
        {
            filters.Add(new HandleErrorAttribute());
        }

        public static void RegisterRoutes(RouteCollection routes)
        {
            routes.IgnoreRoute("{resource}.axd/{*pathInfo}");

            routes.MapRoute(
                "Default",                                              // Route name
                "{controller}/{action}/{id}",                           // URL with parameters
                new { controller = "Home", action = "Index", id = "" }  // Parameter defaults
            );
        }

        protected void Application_Start(object sender, EventArgs e)
        {
            SiteMapManager.SiteMaps.Register<XmlSiteMap>("Web", sitmap => sitmap.LoadFrom("~/Web.sitemap"));

            AreaRegistration.RegisterAllAreas();
            AutoMapperConfiguration.Configure();

            RegisterRoutes(RouteTable.Routes);
            RegisterGlobalFilters(GlobalFilters.Filters);
        }

        /// <summary>
        /// Fired when a new user visits the application Web site. 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void Session_Start(object sender, EventArgs e)
        {
            // Authorize user and add current user to session 
            if (!SecurityHelper.Authenticate())
            {
                throw new AuthorizeException();
            }

            AutoMapperConfiguration.Configure();
        }

        /// <summary>
        /// Fired when an application request is received. It's the first 
        /// event fired for a request, which is often a page request (URL) that a user enters. 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void Application_BeginRequest(object sender, EventArgs e)
        {
            ISession session = SessionProvider.GetNewSession();
            CurrentSessionContext.Bind(session);
        }

        protected void Application_EndRequest(object sender, EventArgs e)
        {
            CurrentSessionContext.Unbind(SessionProvider.GetSessionFactory()).Close();
        }
    }
}
