﻿/*
* This script extend the telerik default functions to follow PIxD.
* All Telerik controls must be loaded
*/
$(document).ready(function () {
    // Verify if the telerik extentions are enabled
    if ($.telerik && $.NVS && $.NVS.loadTelerikExtensions) {
        var $t = $.telerik;
        // First update Telerik's functions
        // DatePicker
        if ($t.calendar != null && $t.calendar != undefined && $t.calendar.views != null && $t.calendar.views != undefined) {
            /**
            * Function to put the date format as a tooltip on DatePicker
            */
            if ($t.datepicker) {
                $.extend($t.datepicker, {
                    fixToolTip: function (context) {
                        $('.t-datepicker', context).each(function () {
                            var datePicker = $('input', this).data('tDatePicker');
                            datePicker.$element.attr('title', datePicker.format);
                        });
                    }
                });
            }

            /*** DatePicker /Calendar Changes ***/
            // Add the week number on calendar
            $t.calendar.views[0].body = function (viewedMonth, minDate, maxDate, selectedDate, urlFormat, dates) {

                var html = (new $t.stringBuilder())
			        .cat('<thead><tr class="t-week-header">');

                // week number header
                html.cat('<th>&nbsp;</th>');

                var firstDayIndex = $t.cultureInfo.firstDayOfWeek;
                var days = $t.cultureInfo.days;
                var abbrDays = $t.cultureInfo.abbrDays;

                days = days.slice(firstDayIndex).concat(days.slice(0, firstDayIndex));
                abbrDays = abbrDays.slice(firstDayIndex).concat(abbrDays.slice(0, firstDayIndex));

                for (var i = 0; i < 7; i++) {
                    html.cat('<th scope="col" abbr="')
                        .cat(abbrDays[i])
                        .cat('" title="')
                        .cat(days[i])
                        .cat('">')
                    // NVS:use 2 letters for day abbreviations start
			            .cat($t.cultureInfo.days[i].substr(0, 2))
                    // NVS:use 2 letters for day abbreviations end
                        .cat('</th>');
                }

                html.cat('</tr></thead><tbody>');

                var currentDayInCalendar = $t.datetime.firstVisibleDay(viewedMonth);

                var month = viewedMonth.month();

                var selectedDateInViewedMonth = selectedDate === null ? false :
                                                viewedMonth.year() == selectedDate.year();
                var cellClass;

                for (var weekRow = 0; weekRow < 6; weekRow++) {

                    html.cat('<tr>');
                    // Inserting the week number on calendar
                    var monday = new Date(currentDayInCalendar.value.getTime() + 1000 * 60 * 60 * 24);
                    html.cat('<td class="t-week-number">' + $.NVS.getWeekNumber(monday) + '</td>');

                    for (var day = 0; day < 7; day++) {
                        cellClass =
			            currentDayInCalendar.month() != month ? 't-other-month' :
			            (selectedDateInViewedMonth
			            && currentDayInCalendar.month() == selectedDate.month()
			            && currentDayInCalendar.date() == selectedDate.date()) ? ' t-state-selected' : '';

                        html.cat('<td')
			                .catIf(' class="' + cellClass + '"', cellClass)
			                .cat('>');

                        if ($t.calendar.isInRange(currentDayInCalendar.toDate(), minDate.toDate(), maxDate.toDate())) {
                            html.cat('<a href="')
                            var url = '#';
                            if (urlFormat) {
                                url = $t.calendar.formatUrl(urlFormat, currentDayInCalendar);
                                if (dates && !$t.calendar.isInCollection(currentDayInCalendar, dates)) {
                                    url = '#';
                                }
                            }

                            html.cat(url)
			                    .cat('" class="t-link')
			                    .cat(url != '#' ? ' t-action-link' : '')
                                .cat('" title="')
                                .cat($t.datetime.format(currentDayInCalendar.toDate(), $t.cultureInfo.longDate))
			                    .cat('">')
			                    .cat(currentDayInCalendar.date())
			                    .cat('</a>');
                        } else {
                            html.cat('&nbsp;');
                        }
                        html.cat('</td>');

                        $t.datetime.modify(currentDayInCalendar, $t.datetime.msPerDay);
                    }

                    html.cat('</tr>');
                }
                html.cat('</tbody>');
                return html.string();
            };
        }

        /** 
        * This code implements a custom toolbar on Grid Control
        * 
        * Native Methods overrided:
        * - $.telerik.grid.prototype.rowClick
        * - $.telerik.grid.prototype.bindTo
        * - $t.grid.prototype.headerClick
        * - $t.grid.prototype.pagerClick
        * - $t.grid.prototype.refreshClick
        */
        if ($t.grid && $t.grid != null) {

            $.extend($t.grid.prototype, {
                extensionsLoaded: false,
                commandStatus: null,
                $gridToolbarItemAddRow: null,
                $gridToolbarItemEditRow: null,
                $gridToolbarItemSaveRow: null,
                $gridToolbarItemDeleteRow: null,
                $contextMenu: null,
                contextMenuConfig: null,
                autoExpandable: false,
                pendingInfoMessages: new Array(),

                setUpCustomEvents: function () {
                    if (this.selectable) {
                        var tbody = this.$tbody[0];
                        var grid = this;
                        var nonSelectableRows = 'tr:not(.t-grouping-row,.t-detail-row,.t-no-data)';
                        this.$tbody.delegate(nonSelectableRows, 'dblclick', function (e) {
                            if (this.parentNode == tbody)
                                grid.rowDoubleClick(e);
                        });
                    }
                },

                // Registering and binding all toolbar buttons on Editing Mode
                setUpToolbar: function () {

                    if (!this.editing.mode) return;

                    this.$gridToolbarItemAddRow = $('> .t-grid-toolbar .add', this.element);
                    this.$gridToolbarItemEditRow = $('> .t-grid-toolbar .edit', this.element);
                    this.$gridToolbarItemSaveRow = $('> .t-grid-toolbar .save', this.element);
                    this.$gridToolbarItemDeleteRow = $('> .t-grid-toolbar .delete', this.element);

                    if (this.$gridToolbarItemAddRow.length == 0 || this.$gridToolbarItemSaveRow.length == 0 || this.$gridToolbarItemDeleteRow.length == 0) {
                        throw new Error("The toolbar buttons edit, save and delete are required.");
                        return;
                    }

                    if (this.editing.mode == 'InLine' && this.$gridToolbarItemEditRow.length == 0) {
                        throw new Error("The toolbar should have one edit button.");
                        return;
                    }

                    this.enableDisableToolbarItems();
                    this.extensionsLoaded = true;
                },

                gridToolbarItemAddRowClick: function (e) {

                    var currentGrid = $(this).closest('.t-grid').data('tGrid');

                    if (!currentGrid.validate()) return;

                    if (currentGrid.getSelectedRow() != null) {
                        currentGrid.getSelectedRow().removeClass('t-state-selected');
                    }

                    currentGrid.addRow();
                    $('> tr.t-grid-new-row', currentGrid.$tbody).addClass('t-state-selected');


                    if ($.fn.contextMenu)
                        $(currentGrid.getSelectedRow()).contextMenu({ menu: currentGrid.$contextMenu.attr('id') }, currentGrid.contextMenuItemClick);

                    currentGrid.commandStatus = "inserting";
                    currentGrid.enableDisableToolbarItems();
                },

                gridToolbarItemEditRowClick: function (e) {
                    var currentGrid = $(this).closest('.t-grid').data('tGrid');

                    var $selectedRow = currentGrid.getSelectedRow();

                    // if just clicked on button without select an item
                    if ($selectedRow == null) return;

                    currentGrid.editRow($selectedRow);
                    currentGrid.commandStatus = "editing";
                    currentGrid.enableDisableToolbarItems();
                },

                gridToolbarItemSaveRowClick: function (e) {
                    var currentGrid = $(this).closest('.t-grid').data('tGrid');

                    if (currentGrid.editing.mode == 'InCell' && currentGrid.hasChanges()) {
                        $.NVS.clearErrorSummary();

                        if (currentGrid.commandStatus == "inserting")
                            currentGrid.pendingInfoMessages.push($.NVS.localization.TelerikGridAddedRow);

                        currentGrid.pendingInfoMessages.push($.NVS.localization.TelerikGridSavedAllRows);

                        currentGrid.submitChanges();
                    }
                    else {
                        if (currentGrid.hasPendingCommands()) {
                            currentGrid.processPendingCommands();
                            return;
                        }
                    }
                },

                gridToolbarItemDeleteRowClick: function (e) {
                    var currentGrid = $(this).closest('.t-grid').data('tGrid');
                    var $selectedRow = currentGrid.getSelectedRow();

                    if ($selectedRow == null) return;

                    if (confirm(currentGrid.localization.deleteConfirmation)) {
                        if (currentGrid.commandStatus == "inserting") {
                            // remove only the line on grid
                            if (currentGrid.changeLog)
                                currentGrid.changeLog.inserted = [];
                            
                            currentGrid.cancel($selectedRow, currentGrid.dataItem($selectedRow));
                        }
                        else {
                            // deletes the data on server
                            currentGrid.pendingInfoMessages.push($.NVS.localization.TelerikGridDeletedRow);

                            currentGrid.sendValues(currentGrid.dataItem($selectedRow), 'deleteUrl');
                        }

                        // finishes the editing
                        $.NVS.clearErrorSummary();
                        currentGrid.resetCommands();
                    }
                },

                hasPendingCommands: function () {
                    return this.commandStatus != null;
                },

                processPendingCommands: function () {
                    var $selectedRow = this.getSelectedRow();
                    $.NVS.clearErrorSummary();

                    switch (this.commandStatus) {
                        // must have a row selected                                                                                                                                                                
                        case "inserting":
                            // validate and update
                            if (!this.insertRow($selectedRow)) {
                                $.NVS.panelSetLayout.resizeAll();
                                return;
                            }

                            this.pendingInfoMessages.push($.NVS.localization.TelerikGridAddedRow);
                            break;

                        case "editing":
                            // validate and update
                            if (!this.updateRow($selectedRow)) {
                                $.NVS.panelSetLayout.resizeAll();
                                return;
                            }

                            this.pendingInfoMessages.push($.NVS.localization.TelerikGridSavedRow);
                            break;

                        default:
                            // Error on processPendingCommands, should not came here
                            throw new Error();
                    }
                },

                postSuccessWithErrors: function (data) {
                    this.pendingInfoMessages = new Array();
                    $.NVS.displayErrors(data.modelState);
                },

                postSuccessWithNoErrors: function () {
                    if (this.extensionsLoaded) {
                        this.resetCommands();

                        for (var message = this.pendingInfoMessages.pop(); message != null; message = this.pendingInfoMessages.pop()) {
                            $.NVS.appendInlineInfoMessage(message);
                        }
                    }
                },

                // this function is called when the server respond and has no error
                resetCommands: function () {
                    this.commandStatus = null;
                    this.enableDisableToolbarItems();
                },

                cancelCurrentPendingCommands: function () {
                    if (!this.editing.mode || !this.hasPendingCommands()) return;

                    if (confirm($.NVS.localization.TelerikSaveCurrentAction) && this.hasPendingCommands()) {
                        this.processPendingCommands();
                    }
                },

                getSelectedRow: function () {
                    var selectedRow = $('> tr.t-state-selected', this.$tbody);
                    if (selectedRow.length == 0)
                        return null;
                    else
                        return selectedRow;
                },

                getAllToolbarItems: function () {
                    return $('> .t-grid-toolbar input', this.element);
                },

                enableDisableToolbarItems: function () {
                    switch (this.commandStatus) {
                        case "inserting":
                            if (this.editing.mode != 'InCell') {
                                // Edit
                                this.disableToolbarItem(this.$gridToolbarItemEditRow);

                                if (this.$contextMenu)
                                    this.$contextMenu.disableContextMenuItems('#addRow,#editRow,#deleteRow');

                                // Save
                                this.enableToolbarItem(this.$gridToolbarItemSaveRow);
                            }

                            // Add
                            this.disableToolbarItem(this.$gridToolbarItemAddRow);
                            break;

                        case "editing":
                            // Edit
                            this.disableToolbarItem(this.$gridToolbarItemEditRow);

                            if (this.$contextMenu)
                                this.$contextMenu.disableContextMenuItems('#addRow,#editRow,#deleteRow');

                            // Add
                            this.disableToolbarItem(this.$gridToolbarItemAddRow);

                            // Save
                            this.enableToolbarItem(this.$gridToolbarItemSaveRow);

                            break;

                        // Enable                                                                                                                                                             
                        default:
                            // if the editing mode is turned off
                            if (!this.editing.mode)
                                return;

                            if (this.$contextMenu)
                                this.$contextMenu.enableContextMenuItems('#addRow,#editRow,#deleteRow');

                            if (this.editing.mode != 'InCell') {
                                // Edit
                                this.enableToolbarItem(this.$gridToolbarItemEditRow);

                                // Save
                                this.disableToolbarItem(this.$gridToolbarItemSaveRow);
                            }
                            else {
                                // Save
                                this.enableToolbarItem(this.$gridToolbarItemSaveRow);
                            }

                            // Add
                            this.enableToolbarItem(this.$gridToolbarItemAddRow);

                            // Delete
                            this.enableToolbarItem(this.$gridToolbarItemDeleteRow);

                            break;
                    }
                },

                enableToolbarItem: function ($item) {
                    if ($item.hasClass('disabled')) {
                        $item.removeClass('disabled');
                    }

                    if (!$.NVS.isClickEventAttached($item[0])) {
                        if ($item.hasClass('edit'))
                            $item.click(this.gridToolbarItemEditRowClick);
                        else if ($item.hasClass('save')) {
                            //$item.click(this.gridToolbarItemSaveRowClick);
                            $item.click($t.stopAll(this.gridToolbarItemSaveRowClick));
                        }
                        else if ($item.hasClass('add'))
                            $item.click(this.gridToolbarItemAddRowClick);
                        else if ($item.hasClass('delete'))
                            $item.click(this.gridToolbarItemDeleteRowClick);
                    }
                },

                disableToolbarItem: function ($item) {
                    $item.unbind();
                    $item.addClass('disabled');
                },

                rowDoubleClick: function (e) {
                    var $target = $(e.target);
                    var currentGrid = $target.closest('.t-grid').data('tGrid');

                    if (currentGrid.editing.mode)
                        currentGrid.$gridToolbarItemEditRow.trigger('click', e);

                    if (currentGrid.onRowDoubleClick) {
                        var $target = $(e.target);
                        if (!$target.is(':button,a,:input')) {
                            e.stopPropagation();
                            var $row = $target.closest('tr')
                                    .addClass('t-state-selected')
                                    .siblings()
                                    .removeClass('t-state-selected')
                                    .end();
                            $t.trigger(this.element, 'rowDoubleClick', { row: $row[0] });
                        }
                    }
                },

                /**
                * Create/attach a Context Menu to Grid
                */
                createContextMenu: function () {
                    this.contextMenuConfig = (arguments.length > 0) ? arguments[0] : { addRow: true, editRow: true, deleteRow: true };

                    var html = '<ul id="' + this.element.id + 'ContextMenu" class="contextMenu">';

                    if (this.contextMenuConfig.addRow)
                        html += '<li class="icon add"><a href="#addRow">Add</a></li>';

                    if (this.contextMenuConfig.editRow)
                        html += '<li class="icon edit"><a href="#editRow">Edit</a></li>';

                    if (this.contextMenuConfig.deleteRow)
                        html += '<li class="icon delete"><a href="#deleteRow">Delete</a></li>';

                    if (this.contextMenuConfig.addRow || this.contextMenuConfig.editRow || this.contextMenuConfig.deleteRow)
                        html += '<li class="icon separator">&nbsp;</li>';

                    if (this.contextMenuConfig.customItems && $.isArray(this.contextMenuConfig.customItems)) {
                        for (var i = 0; i < this.contextMenuConfig.customItems.length; i++) {
                            var item = this.contextMenuConfig.customItems[i];
                            html +=
                        '<li class="icon ' + ((item.cssClass) ? item.cssClass : '') + '">' +
                            ((item.callback) ? '<a href="#customItem' + i + '">' + item.label + '</a>' : item.label) +
                        '</li>';
                        }
                    }

                    html += '</ul>';

                    $('body').append(html);

                    this.$contextMenu = $('#' + this.element.id + 'ContextMenu');
                },

                contextMenuItemClick: function (action, selectedRow, position) {
                    var currentGrid = $(selectedRow).closest('.t-grid').data('tGrid');
                    var $selectedRow = currentGrid.getSelectedRow();

                    if ($selectedRow != null) {
                        $selectedRow.removeClass('t-state-selected');
                    }

                    $(selectedRow).addClass('t-state-selected');

                    switch (action) {
                        case "deleteRow":
                            currentGrid.$gridToolbarItemDeleteRow.trigger('click');
                            break;

                        case "addRow":
                            currentGrid.$gridToolbarItemAddRow.trigger('click');
                            break;

                        case "editRow":
                            currentGrid.$gridToolbarItemEditRow.trigger('click');
                            break;

                        default:
                            if ($.isArray(currentGrid.contextMenuConfig.customItems)) {
                                var item = currentGrid.contextMenuConfig.customItems[parseInt(action.substring(10))];
                                item.callback();
                            }
                            break;
                    }
                },

                attachCustomEvent: function (eventName, eventHandler) {
                    switch (eventName) {
                        case 'onRowDoubleClick':
                            this.onRowDoubleClick = eventHandler;
                            $t.bind(this, {
                                rowDoubleClick: this.onRowDoubleClick
                            });
                            break;
                        default:
                            alert('Event ' + eventName + ' is not handleable');
                            break;
                    }
                }
            });
        }


        /**
        * This code fixs the issue with the list from DropDownList and the jQuery Layout Plugin
        * Native Methods overrided:
        * - $.telerik.list.getZIndex
        */
        $.extend($.telerik.list, {
            getZIndex: function (element) {
                var zIndex = 'auto';
                $(element).parents().andSelf().each(function () { //get element 
                    zIndex = $(this).css('zIndex');
                    if (Number(zIndex)) {
                        zIndex = Number(zIndex) + 1;
                        return false;
                    }
                });
                return zIndex + 1;
            }
        });

        /**
        * Function to load ComboBox and DropDownList with a different URL and parameters
        */
        function comboBoxCustomLoad(combobox, params) {
            if (combobox.ajax.selectUrl.indexOf('?') != -1) {
                combobox.ajax.selectUrl = combobox.ajax.selectUrl.substr(0, combobox.ajax.selectUrl.indexOf('?'));
            }
            combobox.ajax.selectUrl += '?' + params;

            if (!combobox.dropDown)
                combobox.text('');

            combobox.reload();
        }
    } // end if ($.telerik && $.NVS && $.NVS.loadTelerikExtensions)
});

// This code must run after loads everything

$(window).load(function () {
    // Verify if the telerik extentions are enabled
    if ($.telerik && $.NVS && $.NVS.loadTelerikExtensions) {
        // Search for all grids 
        $('.t-grid').each(function () {
            var currentGrid = $(this).data('tGrid');

            // set up the toolbar
            currentGrid.setUpToolbar();
            currentGrid.setUpCustomEvents();

            /** 
            * This code implements the page number input
            */
            currentGrid.$pager.prepend('<span class="t-pager-input"><input name="pageInput-text" maxlength="3" size="3" type="text" /><a href="#">' + $.NVS.localization.TelerikGridPageNumber + '</a></span>');
            $('.t-pager-input a', currentGrid.$pager).click(function () {
                var currentGrid = $(this).closest('.t-grid').data('tGrid');

//                if (currentGrid.editing != null) {
//                    if (currentGrid.editing && !currentGrid.validate()) {
//                        return;
//                    }
//                }

                if (currentGrid.hasPendingCommands()) {
                    currentGrid.cancelCurrentPendingCommands();
                    return;
                }

                var pageNumber = parseInt($(this).parent().children('input').val());

                if (!isNaN(pageNumber)) {
                    currentGrid.pageTo(pageNumber);
                }
            });

            $.NVS.updateGridsHeight();
        });
        // Fixed Calendar generated by defalt telerik assembly, including week numbers
        $('.t-calendar').each(function () {
            $('table thead tr', this).prepend('<th>&nbsp;</th>');

            for (var i = 0; i < 7; i++) {
                $('table thead tr th:nth-child(' + (i + 2) + ')', this).html($.telerik.cultureInfo.days[i].substr(0, 2));
            }

            var tCalendar = $(this).data('tCalendar');

            var dayOfWeek = tCalendar.viewedMonth.value.getDay();
            if (dayOfWeek < 1)
                dayOfWeek = 6;
            else
                dayOfWeek--;

            var offSet = 1000 * 60 * 60 * 24 * dayOfWeek;

            var firstMonday = new Date(tCalendar.viewedMonth.value.getTime() - offSet);

            $('table tbody tr', this).each(function (i) {
                var $tr = $(this);
                var currentDateOnCalendar = new Date(firstMonday.getTime() + (i * 1000 * 60 * 60 * 24 * 7));
                $tr.prepend('<td class="t-week-number">' + $.NVS.getWeekNumber(currentDateOnCalendar) + '</td>');
            });
        });
        // Fixs all tooltips of all datepicker
        if ($.telerik.datepicker) {
            $.telerik.datepicker.fixToolTip();
        }
    } // end if ($.telerik && $.NVS && $.NVS.loadTelerikExtensions)
});
